using System;
using Microsoft.SPOT;


/// <summary>
/// The CmdSfeer will create a certain Ambiance in the room. The function is activated by setting a certain I/O input 
/// channel. If this button is pressed all specified lights are turned off and all dimmers are set to a predefined value.
/// </summary>
class CmdAmbiance : SwitchButton
{
    // variables
    private Group _IOtoClear;
    private Group _dacToSet;
    private int _dacValue;
    private Boolean isAmbianceEnabled;

    private int[] keepStateIO;
    private int[] keepStateDAC;

    /// <summary>
    /// The constructor CmdAmbiance is called upon the creation of the class.
    /// </summary>
    /// <param name="inputChannel">The input channel to monitor.</param>
    /// <param name="IOtoClear">The I/O channels to clear.</param>
    /// <param name="dacToSet">The DAC channels to set</param>
    /// <param name="dacValue">The value the DAC channels should get</param>
    public CmdAmbiance(int inputChannel,Group IOtoClear,Group dacToSet, int dacValue)
    {
        _IOtoClear = IOtoClear;
        _dacToSet = dacToSet;
        isAmbianceEnabled = false; 
        _dacValue = dacValue;
        createPermanentInstance(this, inputChannel);
    }



    /// <summary>
    /// buttonDown() will perform the actions to be taken when the button is pressed.
    /// </summary>
    public override void buttonDown() 
    {
        if (isAmbianceEnabled)
        {
            isAmbianceEnabled = false;

            for (int i = 0; i < _IOtoClear.getChannels().Length; i++)
            {
                if (keepStateIO[i] == 1) k8000().setIOchannel(_IOtoClear.getChannels()[i]);
            }

            keepStateDAC = new int[_dacToSet.getChannels().Length];
            for (int i = 0; i < _dacToSet.getChannels().Length; i++)
            {
                k8000().outputDACchannel(_dacToSet.getChannels()[i], keepStateDAC[i]);
            }
        }
        else
        {   // The ambiance was not enabled yet. We will do so now.
            isAmbianceEnabled = true;
            // the button has been pressed. Now we have to clear the predefined set of
            // I/O channels.
            keepStateIO = new int[_IOtoClear.getChannels().Length];
            for (int i = 0; i < _IOtoClear.getChannels().Length; i++)
            {
                int channelToCheck = _IOtoClear.getChannels()[i];
                k8000().readIOchannel(channelToCheck);
                keepStateIO[i] = k8000().IO[channelToCheck];
                k8000().clearIOchannel(channelToCheck);
            }

            // the button has been pressed. Now we have to set the predefined number of
            // DAC channels.
            keepStateDAC = new int[_dacToSet.getChannels().Length];
            for (int i = 0; i < _dacToSet.getChannels().Length; i++)
            {
                keepStateDAC[i] = k8000().DAC[_dacToSet.getChannels()[i]];
                k8000().outputDACchannel(_dacToSet.getChannels()[i], _dacValue);
            }
        }
    }
}


/// <summary>
/// helper class to construct a group of chnanels.
/// </summary>
class Group {
    private int[] _channels;

    public Group(params  int[] channels)
    {
        _channels = channels;
    }


    public int[] getChannels()
    {
        return _channels;
    }
}


