using System;
using Microsoft.SPOT;

/// <summary>
/// the CmdToggle provides a toggle button functionality: the state of a channel inverts each
/// time the input button has been pressed.
/// </summary>
class CmdToggle : SwitchButton
{
    // declarations
    private int _outputChannel;


    /// <summary>
    /// The constructor CmdToggle is called upon the creation of the class.
    /// </summary>
    /// <param name="inputChannel">The input channel to monitor</param>
    /// <param name="outputChannel">The output channel to control</param>
    public CmdToggle(int inputChannel, int outputChannel)
    {
        _outputChannel = outputChannel;
        createPermanentInstance(this, inputChannel);
    }


    /// <summary>
    /// Toggle the state of the given channel the moment the button is pressed.
    /// </summary>
    public override void buttonDown() {
        k8000().readIOchannel(_outputChannel);

        int currentState = k8000().IO[_outputChannel];

        if (currentState == 0)
            k8000().setIOchannel (_outputChannel);
        else
            k8000().clearIOchannel (_outputChannel);
    }
}

