using System;
using System.Threading;
using System.Text;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Net;
using GHIElectronics.NETMF.Net.NetworkInformation;
using GHIElectronics.NETMF.Net.Sockets;

public class Telnet
{
    // Our LED eblock is connected to Di5
    static OutputPort LED = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.Di5, false);

    public void runServer ()
    {
        byte[] ip = { 192, 168, 2, 150 };
        byte[] subnet = { 255, 255, 255, 0 };
        byte[] gateway = { 192, 168, 1, 1 };
        byte[] mac = { 0x00, 0x88, 0x98, 0x90, 0xD4, 0xE0 };
        WIZnet_W5100.Enable(SPI.SPI_module.SPI1, (Cpu.Pin)FEZ_Pin.Digital.Di10,
        (Cpu.Pin)FEZ_Pin.Digital.Di7, true);


        NetworkInterface.EnableStaticIP(ip, subnet, gateway, mac);
        NetworkInterface.EnableStaticDns(gateway);


        // Our Listening socket
        Socket server = new Socket(AddressFamily.InterNetwork, SocketType.Stream,ProtocolType.Tcp);
        // Telnet usually uses port 23
        IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, 23);
        server.Bind(localEndPoint);
        // start listening
        server.Listen(1);
        while (true)
        {
            // Wait for a client
            Socket Sock = server.Accept();
            new TelnetProcess(Sock);
        }
    }
    
    
    
    internal sealed class TelnetProcess
    {
        byte[] prompt = Encoding.UTF8.GetBytes("\r\nHOME>");
        private Socket clientSocket;
        bool EchoIsEnabeled = false;


        public TelnetProcess(Socket Sock)
        {
            clientSocket = Sock;
            // Spawn a new thread
            new Thread(Process).Start();
        }


        private void HandleCommand(string cmd)
        {
            string str;
            cmd = cmd.ToUpper();


            String[] segments = cmd.Split(' ');

            switch (segments[0])
            {
                case "SETIO":
                    SwitchButton.interfaceObject.setAllIO();
                    break;
                
                case "CLEARIO":
                    SwitchButton.interfaceObject.clearAllIO();
                    break;



            }



            switch (cmd)
            {
                case "ECHO":
                    EchoIsEnabeled = !EchoIsEnabeled;
                    clientSocket.Send(prompt);
                    str = "EchoIsEnabeled = " + EchoIsEnabeled;
                    clientSocket.Send(Encoding.UTF8.GetBytes(str), 0, str.Length,
                    SocketFlags.None);
                    break;
                case "LED ON":
                    LED.Write(true);
                    clientSocket.Send(prompt);
                    str = "LED is now on";
                    clientSocket.Send(Encoding.UTF8.GetBytes(str), 0, str.Length,
                    SocketFlags.None);
                    break;
                case "LED OFF":
                    LED.Write(false);
                    clientSocket.Send(prompt);
                    str = "LED is now off";
                    clientSocket.Send(Encoding.UTF8.GetBytes(str), 0, str.Length,
                    SocketFlags.None);
                    break;
            }
        }



        private void Process()
        {
            byte[] command = new byte[100];
            int index = 0;
            using (clientSocket)
            {
                clientSocket.Send(prompt);
                while (true)
                {
                    if (clientSocket.Receive(command, index, 1, SocketFlags.None) == 0)
                    {
                        clientSocket.Close();
                        break;
                    }
                    if (command[index] == '\r')
                    {
                        // do we have some data?
                        string cmd = new string(Encoding.UTF8.GetChars(command), 0, index);
                        HandleCommand(cmd);
                        index = 0;
                        clientSocket.Send(prompt);
                    }
                    else if (command[index] >= 32 && command[index] <= 126)
                    {
                        // Echo back
                        if (EchoIsEnabeled)
                            clientSocket.Send(command, index, 1, SocketFlags.None);
                        index++;
                        if (index >= command.Length)
                        {
                            Debug.Print("We have too much data!");
                            index = 0;//dump it all
                        }
                    }
                }
            }
        }
    }
}