﻿using Microsoft.SPOT;


/**************************************************************************************************************
 * Home Domotic System
 * 
 * Main Program
 * 
 * 
 * This program will control the Velleman K8000 Interface card in a home domotic system. You can easily create
 * a custom application by modifying this Program.cs file.
 * 
 * 
 * Copyright (c) 2012   Vanderhaegen Bart
 * bartv@telenet.be
 * 
 ***************************************************************************************************************/
public class Program
{
    public static void Main()
    {
        /********************************************************************
         * Step 1: declaration of the number of K8000 cards. This code is 
         * required, otherwise the program will crash.
         * ******************************************************************/
        // Specify the number of K8000 cards
        SwitchButton.numberOfK8000Cards = 1;

        

        /*********************************************************************
         * Step 2: (Optional) Set the parameters for the several Cmd objects.
         * The given times are always specified in milliseconds.
         * 
         * You don't need to set these parameters (they always have a default
         * setting).
         *********************************************************************/
        // Dimmer global specifications
        CmdDimmer.DIMTIME = 5000;                           // max time for dimmer to go from max to 0.
        CmdTimer.DURATION_FOR_SECOND_FUNCTION = 2;          // press time before second function of timer is activated
        
        // PanicButton global specifications
        CmdPanicButton.PRESSING_TIME = 3000;                // press time before panic function is activated
        
        // Security  global specifications
        CmdSecurity.MINIMAL_BURN_TIME_sec = 1;              // minimal burn time for a light in the security timer
        CmdSecurity.MAXIMAL_BURN_TIME_sec = 30;              // maximal burn time for a light in the security timer
        CmdSecurity.MINIMAL_PRESS_TIME_ms = 5000;           // minimal press time to activate security function.
        CmdSecurity.RANDOM_CHANCE = 10000;                  // set the chance a certain I/O will be activated (Chance=1/RANDOM_CHANCE. Evaluated 5-10 times per second).
        
        // RTC timer/NTP global specifications
        CmdRTC.ip = new byte[] { 192, 168, 2, 150 };                        // fixed IP adres for your FEZ
        CmdRTC.subnet = new byte[] { 255, 255, 255, 0 };                    // used subnet
        CmdRTC.gateway = new byte[] { 192, 168, 2, 1 };                     // used gateway
        CmdRTC.mac = new byte[] { 0x00, 0x88, 0x98, 0x90, 0xD4, 0xE0 };     // used MAC address
        CmdRTC.ntpHost = new byte[] { 195, 130, 132, 18 };                  // used NTP server (IP address)
        CmdRTC.dns = new byte[] { 192, 168, 2, 1 };                         // used DNS server
        CmdRTC.timeOffset = 3600;

        // Setting up the Debug environment
        Debug.EnableGCMessages(false);                      // enable only Debug messages from the Home Automation



        /********************************************************************
         * Step 3: define your functionalities.
         * 
         * add a line with a new statement for each object you want to define.
         * 
         * The sample here has some constants which hold the channel numbers.
         * This is not required, but only done for a better readibility.
         * 
         * See more information about the classes on my website:
         * http://www.bvsystems.be/k8000homeAutomationModules.php
         * 
         * ******************************************************************/

        // Function #9
        const int BTN_START_ALL = 1;
        new CmdStartAll(BTN_START_ALL);


        // Function #10
        const int BTN_STOP_ALL = 2;
        new CmdStartAll(BTN_STOP_ALL);


        // Function #11
        const int BTN_TOGGLE_1 = 3;
        const int IO_TOGGLE_LIGHT = 4;
        new CmdToggle(BTN_TOGGLE_1, IO_TOGGLE_LIGHT);


        // Function #8
        const int BTN_TIMER1 = 1;
        const int IO_TIMEROUT_1 = 16;
        new CmdTimer(BTN_TIMER1, IO_TIMEROUT_1, 3, false);


        // Function #7
        // - REQUIRES A FEZ CONNECT SHIELD!!! 
        // - SET NETWORK SETTINGS IN STEP 2!!!
        const int IO_TIMED_OUTPUT = 2;
        TimedEvents myEvents = new TimedEvents();
        myEvents.add("08:00", IO_TIMED_OUTPUT, false);
        myEvents.add("16:00", IO_TIMED_OUTPUT, true);
        new CmdTimedEvents(myEvents);


        // Function #6
        // - SEE GLOBAL SETTINGS IN STEP 2!!!
        int BTN_SECURITY = 1;
        new CmdSecurity(BTN_SECURITY, true, false, 10, 11, 12, 13, 14, 15, 16);


        // Function #5
        // - REQUIRES A FEZ CONNECT SHIELD!!! 
        // - SET NETWORK SETTINGS IN STEP 2!!!
        new CmdRTC(12);


        // Function #4
        const int BTN_PANIC = 1;
        new CmdPanicButton(BTN_PANIC, 16, 15, 14, 13);


        // Function #3
        new CmdLogger(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16);


        // Function #2
        const int IO_DIMMER_BUTTON = 1;
        const int DAC_DIMMER_OUTPUT = 1;
        new CmdDimmer(IO_DIMMER_BUTTON, DAC_DIMMER_OUTPUT);


        // Function #1
        const int BTN_AMBIANCE = 2;
        new CmdAmbiance(BTN_AMBIANCE,
                        new Group(16, 15, 14, 13),
                        new Group(1, 2, 3, 4, 5, 6, 7 ,8),
                        40);



        /********************************************************************
         * Step 4: execute the program. This part of the program will run
         * for ever. You don't need to modify this code.
         * ******************************************************************/
        SwitchButton.executeAll();
    }
}